unit Cob_Format;

interface

{$MINENUMSIZE 1}
type
// Every COB begins with this header
  PCobHeader = ^TCobHeader;
  TCobHeader = packed record
    // Specifies the COB version
    Version : longword;
    // The number of script Procs in this COB file
    NumberOfProcs : longword;
    // The number of pieces declared in this COB file
    NumberOfPieces : Longword;
    // The size in bytes of all the script Procs
    LengthOfAllProcs : longword;
    // The number of static variables declared in this COB
    NumberOfStaticVars : longword;
    // This is always 0
    Always_0 : longword;
    // Offset to an array of indecies. These indecies point to the script
    // Procs of the COB. The array consists of UINT32[ NumberOfProcs ]
    OffsetToProcPointerArray : longword;
    // Offset to an array of indecies. These indecies point to the names of
    // the script Procs in the name array. The array consists of UINT32[ NumberOfScripts ]
    OffsetToProcNameOffsetArray : longword;
    // Offset to an array of indecies. These indecies point to the names of
    // the pieces in the name array. The array consists of UINT32[ NumberOfPieces ]
    OffsetToPieceNameOffsetArray : longword;
    // Offset to the first Proc in the COB
    OffsetToFirstProc : LongInt;
    // Offset to the name array
    OffsetToNameArray : longword;
{ // only used in TA:K which isnt supported
    // Offset to an array of indecies. These indecies point to the names of
    // the sounds in the name array. The array consists of UINT32[ NumberOfSounds ]
    OffsetToSoundNameArray : longword;
    // The number of sounds used in the script
    NumberOfSounds : longword;
}
  end; {TCobHeader}

const
  // Indicates that the COB is from Total Annihilation
  CobVersion_TotalAnnihilation = $00000004;
  // Indicates that the COB is from TA:Kingdoms
  CobVersion_Kingdoms = $00000006;
  // Indicates that the COB is from TA:Kingdoms
  CobVersion_TotalAnnihilation_Cobalt = $00000007;
// Known opcodes
Const
  Opcode_MovePieceWithSpeed           =  $10001000;
  Opcode_TurnPieceWithSpeed           =  $10002000;
  Opcode_StartSpin                    =  $10003000;
  Opcode_StopSpin                     =  $10004000;
  Opcode_ShowPiece                    =  $10005000;
  Opcode_HidePiece                    =  $10006000;
  Opcode_CachePiece                   =  $10007000;
  Opcode_DontCachePiece               =  $10008000;
  Opcode_DontShadow                   =  $1000A000;
  Opcode_MovePieceNow                 =  $1000B000;
  Opcode_TurnPieceNow                 =  $1000C000;
  Opcode_DontShade                    =  $1000E000;
  Opcode_EmitSfx                      =  $1000F000;
  Opcode_WaitForTurn                  =  $10011000;
  Opcode_WaitForMove                  =  $10012000;
  Opcode_Sleep                        =  $10013000;
  Opcode_PushConstantOntoStack        =  $10021001;
  Opcode_PushLocalVariableOntoStack   =  $10021002;
  Opcode_PushStaticVariableOntoStack  =  $10021004;
  Opcode_StackAllocate                =  $10022000;
  Opcode_SetLocalVariable             =  $10023002;
  Opcode_SetStaticVariable            =  $10023004;
  Opcode_PopStack                     =  $10024000;
  Opcode_Add                          =  $10031000;
  Opcode_Subtract                     =  $10032000;
  Opcode_Multiply                     =  $10033000;
  Opcode_Divide                       =  $10034000;
  Opcode_BitwiseAnd                   =  $10035000;
  Opcode_BitwiseOr                    =  $10036000;
  Opcode_Unknown1                     =  $10039000;
  Opcode_Unknown2                     =  $1003A000;
  Opcode_Unknown3                     =  $1003B000;
  Opcode_Random                       =  $10041000;
  Opcode_GetUnitValue                 =  $10042000;
  Opcode_GetFunctionResult            =  $10043000;
  Opcode_LessThan                     =  $10051000;
  Opcode_LessThanOrEqual              =  $10052000;
  Opcode_GreaterThan                  =  $10053000;
  Opcode_GreaterThanOrEqual           =  $10054000;
  Opcode_Equal                        =  $10055000;
  Opcode_NotEqual                     =  $10056000;
  Opcode_And                          =  $10057000;
  Opcode_Or                           =  $10058000;
  Opcode_Not                          =  $1005A000;
  Opcode_StartScript                  =  $10061000;
  Opcode_CallScript                   =  $10062000;
  Opcode_JumpToOffset                 =  $10064000;
  Opcode_Return                       =  $10065000;
  Opcode_JumpToOffsetIfFalse          =  $10066000;
  Opcode_Signal                       =  $10067000;
  Opcode_SetSignalMask                =  $10068000;
  Opcode_Explode                      =  $10071000;
//  Opcode_PlaySound                    =  $10072000;
//  Opcode_MapCommand                   =  $10073000;
  Opcode_SetUnitValue                 =  $10082000;
  Opcode_AttachUnit                   =  $10083000;
  Opcode_DropUnit                     =  $10084000;
  // Cobbler signature - First 4 bytes are 'Buil'; continues for  bytes
  Opcode_CobblerCrap                  =  $6c697542;
  // Fake opcodes used by Scriptor
  Opcode_ControlBlock                 =  $90000000;

implementation

end.
